@@ -2,24 +2,30 @@
 import builtins
 import json
 import os
+from collections import OrderedDict
 from csv import DictWriter
 from functools import reduce
 from hashlib import md5
 from inspect import signature, getfullargspec
 from multiprocessing import pool
 
+import click
 import cloudinary
-from jinja2 import Environment
-
+from jinja2 import Environment, FileSystemLoader
+from docstring_parser import parse
 from cloudinary_cli.defaults import logger, TEMPLATE_FOLDER
 
 not_callable = ('is_appengine_sandbox', 'call_tags_api', 'call_context_api', 'call_cacheable_api', 'call_api',
-                'call_metadata_api', 'call_json_api', 'only', 'transformation_string'
-                'account_config', 'reset_config', 'upload_large_part', 'upload_image', 'upload_resource')
+                'call_metadata_api', 'call_json_api', 'only', 'transformation_string', 'account_config',
+                'reset_config', 'upload_large_part', 'upload_image', 'upload_resource')
 
 BLOCK_SIZE = 65536
 
 
+class ConfigurationError(Exception):
+    pass
+
+
 def etag(fi):
     file_hash = md5()
     with open(fi, 'rb') as f:
@@ -36,21 +42,29 @@ def is_builtin_class_instance(obj):
 
 
 def get_help_str(module, block_list=(), allow_list=()):
-    funcs = list(filter(
-        lambda f:
-        callable(module.__dict__[f])
-        and not is_builtin_class_instance(module.__dict__[f])
-        and f[0].islower()
-        and (f not in block_list and block_list)
-        and (f in allow_list or not allow_list),
-        module.__dict__.keys()))
+    funcs = {}
+    for f in module.__dict__.keys():
+        if callable(module.__dict__[f]) \
+                and not is_builtin_class_instance(module.__dict__[f]) \
+                and f[0].islower() \
+                and (f not in block_list and block_list) \
+                and (f in allow_list or not allow_list):
+            funcs[f] = {"params": ", ".join(signature(module.__dict__[f]).parameters),
+                        "desc": parse(module.__dict__[f].__doc__).short_description}
 
-    template = "{0:" + str(len(max(funcs, key=len)) + 1) + "}({1})"  # Gets the maximal length of the functions' names
+    funcs = OrderedDict(sorted(funcs.items()))
 
-    return '\n'.join([template.format(f, ", ".join(list(signature(module.__dict__[f]).parameters))) for f in funcs])
+    # Gets the max length of the functions' names
+    template = "{0:" + str(len(max(funcs.keys(), key=len)) + 1) + "}({1:30} {2}"
+
+    return '\n'.join(
+        [
+            template.format(f, p["params"] + ")", p["desc"] if p["desc"] is not None else "")
+            for f, p in funcs.items()
+        ])
 
 
-def print_help(api, block_list=not_callable, allow_list=()):
+def print_api_help(api, block_list=not_callable, allow_list=()):
     logger.info(get_help_str(api, block_list=block_list, allow_list=allow_list))
 
 
@@ -62,13 +76,22 @@ def log_exception(e, message=None):
 
 def load_template(language, template_name):
     filepath = os.path.join(TEMPLATE_FOLDER, language, template_name)
+    if not os.path.exists(filepath):
+        logger.error(f"Template: '{template_name}' for language: '{language}' does not exist")
+        return False
     try:
         with open(filepath) as f:
-            template = Environment().from_string(f.read())
+            template = Environment(loader=FileSystemLoader(TEMPLATE_FOLDER)).from_string(f.read())
     except IOError:
         logger.error(f"Failed loading template: '{template_name}' for language: '{language}'")
         raise
-    return template.render(**cloudinary.config().__dict__)
+    try:
+        result = template.render(**cloudinary.config().__dict__)
+    except Exception:
+        logger.error(f"Failed rendering template: '{template_name}' for language: '{language}'")
+        raise
+
+    return result
 
 
 def parse_option_value(value):
@@ -93,7 +116,8 @@ def parse_args_kwargs(func, params):
 
     n_req = n_args - n_defaults
     if len(params) < n_req:
-        raise Exception("Function '{}' requires {} arguments".format(func.__name__, n_req))
+        func_sig = signature(func)
+        raise Exception(f"Function '{func.__name__}{func_sig}' requires {n_req} positional arguments")
     # consume required args
     args = [parse_option_value(p) for p in params[:n_req]]
     kwargs = {}
@@ -183,7 +207,7 @@ def get_command_params(
     if not callable(func):
         raise Exception(f"{params[0]} is not callable.")
 
-    args, kwargs = parse_args_kwargs(func, params[1:]) if len(params) > 1 else ([], {})
+    args, kwargs = parse_args_kwargs(func, params[1:])
 
     kwargs = {
         **kwargs,
@@ -194,6 +218,15 @@ def get_command_params(
     return func, args, kwargs
 
 
+def print_help_and_exit():
+    """
+    Prints help for the current command and exits.
+    """
+    ctx = click.get_current_context()
+    click.echo(ctx.get_help())
+    ctx.exit()
+
+
 def whitelist_keys(data, keys):
     """
     Iterates over a list of dictionaries and keeps only the keys that were specified.
@@ -256,3 +289,23 @@ def normalize_list_params(params):
             normalized_params.append(f)
 
     return normalized_params
+
+
+def chunker(seq, size):
+    """
+    Iterates a sequence in chunks of a given size.
+
+    >>> for group in chunker(['cat', 'dog', 'rabbit', 'duck', 'bird', 'cow', 'gnu', 'fish'], 3):
+    >>>    print(group)
+
+    Produces:
+
+    ['cat', 'dog', 'rabbit']
+    ['duck', 'bird', 'cow']
+    ['gnu', 'fish']
+
+    :param seq: The sequence to iterate.
+    :param size: The size of a single chunk.
+    :return: a single chunk
+    """
+    return (seq[pos:pos + size] for pos in range(0, len(seq), size))
